@@ -1,4 +1,4 @@
-# Copyright 2008-2016 Kouhei Sutou <kou@cozmixng.org>
+# Copyright 2008-2017 Kouhei Sutou <kou@cozmixng.org>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,6 @@ end
 
 require "rbconfig"
 
-require 'mkmf'
 require 'shellwords'
 require 'English'
 require 'pathname'
@@ -46,6 +45,14 @@ class PackageConfig
     end
 
     private
+    def with_config(config, default=nil)
+      if defined?(super)
+        super
+      else
+        default
+      end
+    end
+
     def guess_native_pkg_config
       pkg_config = with_config("pkg-config", ENV["PKG_CONFIG"] || "pkg-config")
       pkg_config = Pathname.new(pkg_config)
@@ -143,6 +150,10 @@ class PackageConfig
     collect_cflags[0].join(" ")
   end
 
+  def cflags_only_other
+    collect_cflags[1].join(" ")
+  end
+
   def libs
     path_flags, other_flags = collect_libs
     (path_flags + other_flags).join(" ")
@@ -202,6 +213,7 @@ class PackageConfig
     end
     all_cflags = normalize_cflags(Shellwords.split(cflags_set.join(" ")))
     path_flags, other_flags = all_cflags.partition {|flag| /\A-I/ =~ flag}
+    path_flags = normalize_path_flags(path_flags, "-I")
     path_flags = remove_duplicated_include_paths(path_flags)
     path_flags = path_flags.reject do |flag|
       flag == "-I/usr/include"
@@ -214,6 +226,24 @@ class PackageConfig
     [path_flags, other_flags]
   end
 
+  def normalize_path_flags(path_flags, flag_option)
+    path_flags.collect do |path_flag|
+      path = path_flag.sub(flag_option, "")
+      prefix = ""
+      case RUBY_PLATFORM
+      when "x86-mingw32"
+        prefix = Dir.glob("c:/msys{32,64,*}").first
+      when "x64-mingw32"
+        prefix = Dir.glob("c:/msys{64,*}").first
+      end
+      if /\A[a-z]:/i === path
+        "#{flag_option}#{path}"
+      else
+        "#{flag_option}#{prefix}#{path}"
+      end
+    end
+  end
+
   def normalize_cflags(cflags)
     normalized_cflags = []
     enumerator = cflags.to_enum
@@ -242,8 +272,9 @@ class PackageConfig
     all_libs = [declaration("Libs")] + all_libs
     all_libs = all_libs.join(" ").gsub(/-([Ll]) /, '\1').split.uniq
     path_flags, other_flags = all_libs.partition {|flag| /\A-L/ =~ flag}
+    path_flags = normalize_path_flags(path_flags, "-L")
     path_flags = path_flags.reject do |flag|
-      /\A-L\/usr\/lib(?:64)?\z/ =~ flag
+      /\A-L\/usr\/lib(?:64|x32)?\z/ =~ flag
     end
     if @msvc_syntax
       path_flags = path_flags.collect do |flag|
@@ -302,20 +333,28 @@ class PackageConfig
   end
 
   def guess_default_path
-    arch_depended_path = Dir.glob('/usr/lib/*/pkgconfig').join(SEPARATOR)
+    arch_depended_path = Dir.glob("/usr/lib/*/pkgconfig")
     default_paths = [
       "/usr/local/lib64/pkgconfig",
+      "/usr/local/libx32/pkgconfig",
       "/usr/local/lib/pkgconfig",
       "/usr/local/libdata/pkgconfig",
       "/opt/local/lib/pkgconfig",
-      arch_depended_path,
+      *arch_depended_path,
       "/usr/lib64/pkgconfig",
+      "/usr/libx32/pkgconfig",
       "/usr/lib/pkgconfig",
       "/usr/libdata/pkgconfig",
       "/usr/X11/lib/pkgconfig",
       "/opt/X11/lib/pkgconfig",
       "/usr/share/pkgconfig",
     ]
+    case RUBY_PLATFORM
+    when "x86-mingw32"
+      default_paths.concat(Dir.glob("c:/msys*/mingw32/lib/pkgconfig"))
+    when "x64-mingw32"
+      default_paths.concat(Dir.glob("c:/msys*/mingw64/lib/pkgconfig"))
+    end
     default_path = default_paths.join(SEPARATOR)
     libdir = ENV["PKG_CONFIG_LIBDIR"]
     default_path = [libdir, default_path].join(SEPARATOR) if libdir
@@ -327,6 +366,7 @@ class PackageConfig
       Dir.glob((pkg_config_prefix + "lib/*/pkgconfig").to_s).join(SEPARATOR)
     [pkg_config_arch_depended_path,
      (pkg_config_prefix + "lib64/pkgconfig").to_s,
+     (pkg_config_prefix + "libx32/pkgconfig").to_s,
      (pkg_config_prefix + "lib/pkgconfig").to_s,
      (pkg_config_prefix + "libdata/pkgconfig").to_s,
      default_path].join(SEPARATOR)
@@ -399,6 +439,10 @@ module PKGConfig
     package_config(pkg).cflags_only_I
   end
 
+  def cflags_only_other(pkg)
+    package_config(pkg).cflags_only_other
+  end
+
   def modversion(pkg)
     package_config(pkg).version
   end
@@ -443,7 +487,9 @@ module PKGConfig
       else
         $LDFLAGS += ' ' + dldflags
       end
-      $CFLAGS += ' ' + cflags(pkg)
+      $CFLAGS += ' ' + cflags_only_other(pkg)
+      $CXXFLAGS += ' ' + cflags_only_other(pkg)
+      $INCFLAGS += ' ' + cflags_only_I(pkg)
     end
     enough_version
   end
